/**
 * Create the chart
 */
var ogma = new Ogma({
  container: 'graph-container',
  options: {
    interactions: { zoom: { onDoubleClick: true } }
  }
});

/**
 * Styling section
 */
var personIcon = '\uf007';
var companyIcon = '\uf1ad';
var isShareholderType = 'IS-SHAREHOLDER';

function getNodeColor(node) {
  var group = node.getData('group');
  if (group === 'purple') {
    return '#67328E';
  }
  if (group === 'green') {
    return '#328E5B';
  }
  return '#DE8B53';
}

function getEdgeColor(edge) {
  if (edge.getData('type') === isShareholderType) {
    return '#89C7D6';
  }
  return '#8E6538';
}

function getEdgeHaloColor(edge) {
  if (edge.getData('type') === isShareholderType) {
    return '#BDF2FF';
  }
  return '#DABB98';
}

ogma.styles.addNodeRule({
  text: {
    minVisibleSize: 0
  },
  color: 'white',
  outerStroke: {
    color: getNodeColor
  },
  icon: {
    font: 'Font Awesome 5 Free',
    content: function(node) {
      if (node.getData('type') === 'company') {
        return companyIcon;
      }
      return personIcon;
    },
    style: 'bold',
    color: getNodeColor
  }
});

ogma.styles.setHoveredNodeAttributes({
  outline: false, // Disabling the shadow on hover
  outerStroke: {
    color: getNodeColor
  },
  text: {
    tip: false
  }
});

ogma.styles.addEdgeRule({
  shape: {
    head: 'arrow'
  },
  color: getEdgeColor
});

ogma.styles.setHoveredEdgeAttributes({
  outline: true,
  color: getEdgeColor,
  halo: getEdgeHaloColor,
  text: {
    backgroundColor: 'rgb(220, 220, 220)',
    minVisibleSize: 0
  }
});

function formatContent(ids) {
  if (!Array.isArray(ids)) {
    return ''; // this is for regular nodes
  }
  return (
    '<br/><br/>Contains:<ul/><li>"' + ids.join('"</li><li/>"') + '"</li></ul>'
  );
}

function createTooltip(title, content, color) {
  return (
    '<div class="arrow"></div>' +
    '<div class="ogma-tooltip-header">' +
    '<span class="title">' +
    title +
    '<span class="line-color" style="background: ' +
    color +
    '">' +
    '&nbsp;' +
    '</span>' +
    formatContent(content) +
    '</span>' +
    '</div>'
  );
}

// Show some info from the hovered node
ogma.tools.tooltip.onNodeHover(
  function(node) {
    // pick the color of the node
    var color = node.getAttribute('outerStroke');
    // Now change the title based on the type of node (grouped or not?)
    var title = node.isVirtual()
      ? 'Grouped Area&nbsp;'
      : 'ID: "' + node.getId() + '"&nbsp;';
    // and pick the content of the node if grouped;
    var content = node.isVirtual() ? node.getData('ids') : node.getId();
    return createTooltip(title, content, color);
  },
  {
    className: 'ogma-tooltip' // tooltip container class to bind to css
  }
);

ogma.parse
  .jsonFromUrl('ownerships.json')
  .then(function(graph) {
    ogma.setGraph(graph);
    return ogma.layouts.force();
  })
  .then(function() {
    return ogma.view.locateGraph({ padding: 120 });
  });

// this is the duration of the animations
var transitionTime = 500;
// Menu at the top right
var form = document.querySelector('#ui form');
form.addEventListener('change', updateUI);

document.querySelector('#layout').addEventListener('click', function(evt) {
  evt.preventDefault();
  ogma.layouts.force();
});

// transformation objects: create them now and use them later
var nodeGrouping = ogma.transformations.addNodeGrouping({
  selector: function(node) {
    return node.getData('type') === 'company';
  },
  groupIdFunction: function(node) {
    return node.getData('group');
  },
  nodeGenerator: function(nodeList, groupId) {
    return {
      id: 'grouped-node-' + groupId,
      attributes: {
        text: groupId + ' Group',
        radius: nodeList.size * 3,
        color: nodeList.getAttribute('icon').color
      },
      // add some content to be picked up later on in the tooltip
      data: {
        group: nodeList.getData('group')[0],
        type: nodeList.getData('type')[0],
        ids: nodeList.getId()
      }
    };
  },
  groupEdges: false, // do not group edges as for now
  enabled: false // create the transformation but do not activate it
});
var edgeGrouping = ogma.transformations.addEdgeGrouping({
  generator: function(edgeList, groupId) {
    var data = edgeList.getData()[0];
    return {
      id: groupId,
      data: data,
      attributes: {
        width: edgeList.size
      }
    };
  },
  enabled: false // create the transformation but do not activate it
});

function updateUI() {
  // Check what it should
  var nodeGrouped = getMode('node-group') === 'group';
  var edgeGrouped = getMode('edge-group') === 'group';

  // instead of working out how to go from state A to state B
  // execute all transformation and for each one workout what to do
  return execute(edgeGrouping, edgeGrouped)
    .then(function() {
      return execute(nodeGrouping, nodeGrouped);
    })
    .then(function() {
      return ogma.layouts.force();
    });
}

function execute(transformation, enable) {
  // avoid extra work if we're already in the state requested
  if (transformation.isEnabled() === enable) {
    return Promise.resolve();
  }
  // if now toggle the state with animation
  if (enable) {
    return transformation.enable(transitionTime);
  }
  return transformation.disable(transitionTime);
}

function getMode(id) {
  var select = form[id];
  var currentMode = Array.prototype.filter.call(select, function(input) {
    return input.checked;
  })[0].value; // IE inconsistency
  return currentMode;
}
